@@ -2,7 +2,7 @@
  *
  * Interface to block cryptos and hash functions
  *
- * $Id: abstract_crypto.h,v 1.24 1999/12/07 00:23:08 nisse Exp $ */
+ * $Id: abstract_crypto.h,v 1.25 2000/04/21 22:29:30 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -178,10 +178,6 @@
 #define MAKE_VERIFIER(a, i) \
 ((a)->make_verifier((a), (i)))
 
-#if 0
-#define MAKE_PUBLIC(a, i) \
-((a)->private2public((a), (i)))
-#endif
 
 /* Combining block cryptos */
 
