@@ -2,7 +2,7 @@
  *
  * Information about ssh channels.
  *
- * $Id: channel.h,v 1.54 2000/03/26 20:20:13 nisse Exp $
+ * $Id: channel.h,v 1.56 2000/04/21 22:29:32 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -81,12 +81,9 @@
        ; put onto the connections resource list.
        (resources object resource_list)
        
-       ; We try to keep the rec_window_size between max_window / 2
-       ; and max_window. 
-       ;; (max_window simple UINT32) 
-
-       ; FIXME: Does the maximum packet sizes apply to complete ssh
-       ; packets, or the data payload?
+       ; NOTE: The channels maximum packet sizes refer to the packet
+       ; payload, i.e. the DATA string in SSH_CHANNEL_DATA and
+       ; SSH_MSG_CHANNEL_EXTENDED_DATA.
 
        (rec_window_size simple UINT32)
        (rec_max_packet simple UINT32)
@@ -212,12 +209,8 @@
        (pending_global_requests struct object_queue)
        
        ; If non-zero, close connection after all active channels have
-       ; died.
-       (pending_close simple int)
-
-       ; FIXME: Perhaps we should use an flag to indicate whether or
-       ; not new channels can be opened?
-       ))
+       ; died, and don't allow any new channels to be opened.
+       (pending_close simple int)))
 */
 
 /* SSH_MSG_GLOBAL_REQUEST */
@@ -336,12 +329,6 @@ void
 channel_start_receive(struct ssh_channel *channel,
 		      UINT32 initial_window_size);
 
-#if 0
-struct lsh_string *prepare_channel_open(struct ssh_connection *connection,
-					int type,
-					struct ssh_channel *channel,
-					const char *format, ...);
-#endif
 struct lsh_string *
 format_channel_open(int type, UINT32 local_channel_number,
 		    struct ssh_channel *channel,
