@@ -1,6 +1,6 @@
 /* client.c
  *
- * $Id: client.c,v 1.90 2000/03/26 20:32:58 nisse Exp $ */
+ * $Id: client.c,v 1.92 2000/04/21 22:29:33 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -21,9 +21,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* FIXME: Why include stdio? */
-/* #include <stdio.h> */
-
 #include "client.h"
 
 #include "abstract_io.h"
@@ -199,10 +196,6 @@ do_client_session_eof(struct ssh_channel *c)
   CAST(client_session, session, c);
   
   close_fd(session->in, 0);
-#if 0
-  close_fd(session->out, 0);
-  close_fd(session->err, 0);
-#endif
 }  
 
 static void
@@ -448,7 +441,7 @@ make_client_session(struct lsh_fd *in,
   self->super.rec_window_size = initial_window;
 
   /* FIXME: Make maximum packet size configurable */
-  self->super.rec_max_packet = SSH_MAX_PACKET;
+  self->super.rec_max_packet = SSH_MAX_PACKET - SSH_CHANNEL_MAX_PACKET_FUZZ;
 
   self->super.request_types = make_alist(0, -1);
 
