@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: client.h,v 1.28 2000/03/07 18:14:07 nisse Exp $ */
+ * $Id: client.h,v 1.29 2000/03/26 20:32:59 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -65,7 +65,7 @@ extern struct command client_io;
 struct ssh_channel *make_client_session(struct lsh_fd *in,
 					struct lsh_fd *out,
 					struct lsh_fd *err,
-					UINT32 max_window,
+					UINT32 initial_window,
 					int *exit_status);
 
 struct command *
