@@ -1,6 +1,6 @@
 /* client_keyexchange.c
  *
- * $Id: client_keyexchange.c,v 1.40 1999/12/12 19:06:13 nisse Exp $ */
+ * $Id: client_keyexchange.c,v 1.41 2000/04/21 22:29:34 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -34,12 +34,6 @@
 
 #include <assert.h>
 
-#if 0
-#define GABA_DEFINE
-#include "client_keyexchange.h.x"
-#undef GABA_DEFINE
-#endif
-
 #include "client_keyexchange.c.x"
 
 /* GABA:
