@@ -316,7 +316,7 @@ make_failure_handler(struct client_userauth_state *state)
 }
 
 static void
-do_userauth_banner(struct packet_handler *self,
+do_userauth_banner(struct packet_handler *self UNUSED,
 		   struct ssh_connection *connection UNUSED,
 		   struct lsh_string *packet)
 {
@@ -348,14 +348,9 @@ do_userauth_banner(struct packet_handler *self,
   lsh_string_free(packet);
 }
 
-static struct packet_handler *make_banner_handler(void)
-{
-  NEW(packet_handler, self);
+static struct packet_handler userauth_banner_handler =
+{ STATIC_HEADER, do_userauth_banner };
 
-  self->handler = do_userauth_banner;
-  
-  return self;
-}
 
 /* GABA:
    (class
@@ -419,7 +414,7 @@ do_client_userauth(struct command *s,
   connection->dispatch[SSH_MSG_USERAUTH_FAILURE]
     = make_failure_handler(state);
   connection->dispatch[SSH_MSG_USERAUTH_BANNER]
-    = make_banner_handler();
+    = &userauth_banner_handler;
 
   assert(LIST_LENGTH(self->methods));
   {
