@@ -132,7 +132,7 @@ handle_connection(struct abstract_write *w,
 }
 
 static void
-do_fail(struct packet_handler *closure,
+do_fail(struct packet_handler *closure UNUSED,
 	struct ssh_connection *connection,
 	struct lsh_string *packet)
 {
@@ -143,16 +143,11 @@ do_fail(struct packet_handler *closure,
   PROTOCOL_ERROR(connection->e, NULL);
 }
 
-struct packet_handler *make_fail_handler(void)
-{
-  NEW(packet_handler, res);
-
-  res->handler = do_fail;
-  return res;
-}
+static struct packet_handler fail_handler =
+{ STATIC_HEADER, do_fail };
 
 static void
-do_unimplemented(struct packet_handler *closure,
+do_unimplemented(struct packet_handler *closure UNUSED,
 		 struct ssh_connection *connection,
 		 struct lsh_string *packet)
 {
@@ -169,13 +164,9 @@ do_unimplemented(struct packet_handler *closure,
   lsh_string_free(packet);
 }
 
-struct packet_handler *make_unimplemented_handler(void)
-{
-  NEW(packet_handler, res);
+static struct packet_handler unimplemented_handler =
+{ STATIC_HEADER, do_unimplemented };
 
-  res->handler = do_unimplemented;
-  return res;
-}
 
 /* GABA:
    (class
@@ -202,17 +193,7 @@ do_exc_protocol_handler(struct exception_handler *s,
 	EXCEPTION_RAISE(self->super.parent, &finish_read_exception);
       }
       break;
-#if 0
-    case EXC_CONNECTION_LOCK:
-      assert(!self->connection->busy);
-      self->connection->busy = 1;
-      break;
 
-    case EXC_CONNECTION_UNLOCK:
-      assert(self->connection->busy);
-      self->connection->busy = 0;
-      break;
-#endif 
     default:
       EXCEPTION_RAISE(self->super.parent, e);
     }
@@ -235,13 +216,17 @@ make_exc_protocol_handler(struct ssh_connection *connection,
 }
 
 struct ssh_connection *
-make_ssh_connection(const char *debug_comment,
+make_ssh_connection(struct address_info *peer,
+		    const char *debug_comment,
 		    struct command_continuation *c,
 		    struct exception_handler *e)
 {
   int i;
 
   NEW(ssh_connection, connection);
+
+  connection->peer = peer;
+  
   connection->debug_comment = debug_comment;
   connection->super.write = handle_connection;
 
@@ -261,7 +246,7 @@ make_ssh_connection(const char *debug_comment,
   
   connection->resources = empty_resource_list();
   
-  connection->rec_max_packet = SSH_MAX_PACKET;
+  connection->rec_max_packet = SSH_MAX_PACKET - SSH_CHANNEL_MAX_PACKET_FUZZ;
   connection->rec_mac = NULL;
   connection->rec_crypto = NULL;
 
@@ -282,14 +267,14 @@ make_ssh_connection(const char *debug_comment,
   
   /* Initialize dispatch */
   connection->ignore = make_ignore_handler();
-  connection->unimplemented = make_unimplemented_handler();
-  connection->fail = make_fail_handler();
+  connection->unimplemented = &unimplemented_handler;
+  connection->fail = &fail_handler;
   
   for (i = 0; i < 0x100; i++)
     connection->dispatch[i] = connection->unimplemented;
 
   connection->dispatch[0] = connection->fail;
-  connection->dispatch[SSH_MSG_DISCONNECT] = make_disconnect_handler();
+  connection->dispatch[SSH_MSG_DISCONNECT] = &disconnect_handler;
   connection->dispatch[SSH_MSG_IGNORE] = connection->ignore;
   connection->dispatch[SSH_MSG_UNIMPLEMENTED] = connection->ignore;
 
