@@ -1,6 +1,6 @@
 /* disconnect.c
  *
- * $Id: disconnect.c,v 1.15 1999/08/22 21:24:08 nisse Exp $ */
+ * $Id: disconnect.c,v 1.17 2000/04/21 22:29:36 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -75,10 +75,5 @@ do_disconnect(struct packet_handler *closure UNUSED,
   EXCEPTION_RAISE(connection->e, &disconnect_exception);
 }
 
-struct packet_handler *make_disconnect_handler(void)
-{
-  NEW(packet_handler, res);
-
-  res->handler = do_disconnect;
-  return res;
-}
+struct packet_handler disconnect_handler =
+{ STATIC_HEADER, do_disconnect };
