@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.93 2000/04/11 20:39:08 nisse Exp $ */
+ * $Id: io.c,v 1.99 2000/04/21 22:29:37 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -55,6 +55,7 @@
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/un.h>
 #include <arpa/inet.h>
 #include <signal.h>
 
@@ -239,6 +240,9 @@ int io_iter(struct io_backend *b)
 	if (!fd->super.alive)
 	  continue;
 
+	/* On systems without poll(), we use jpoll.c to emulate some
+	 * of poll(), but we lack POLLNVAL, POLLPRI and POLLHUP. */
+#ifdef POLLNVAL
 	if (fds[i].revents & POLLNVAL)
 	  {
 	    werror("io.c: poll request on fd %i, for events of type %xi\n"
@@ -247,7 +251,9 @@ int io_iter(struct io_backend *b)
 	    kill_fd(fd);
 	    continue;
 	  }
+#endif /* POLLNVAL */
 
+#ifdef POLLHUP
 	/* FIXME: According to Solaris' man page, POLLHUP is mutually
 	 * exclusive with POLLOUT, but orthogonal to POLLIN.
 	 *
@@ -271,7 +277,9 @@ int io_iter(struct io_backend *b)
 	      }
 	    continue;
 	  }
+#endif /* POLLHUP */
 
+#ifdef POLLPRI
 	if (fds[i].revents & POLLPRI)
 	  {
 	    werror("io.c: Peer is trying to send Out of Band data. Hanging up.\n");
@@ -282,7 +290,7 @@ int io_iter(struct io_backend *b)
 
 	    continue;
 	  }
-
+#endif /* POLLPRI */
 	if (fds[i].revents & POLLOUT)
 	  FD_WRITE(fd);
 
@@ -565,8 +573,12 @@ do_listen_callback(struct io_callback *s,
 		   struct lsh_fd *fd)
 {
   CAST(io_listen_callback, self, s);
-  /* FIXME: ip6 support? */
-  struct sockaddr_in peer;
+
+#if WITH_IPV6
+  struct sockaddr_storage peer;
+#else
+  struct sockaddr peer;
+#endif
 
   socklen_t addr_len = sizeof(peer);
   int conn;
@@ -835,8 +847,8 @@ write_raw_with_poll(int fd, UINT32 length, const UINT8 *data)
 /* Network utility functions */
 
 /* Converts a string port number or service name to a port number.
- * Returns the port number in _host_ byte order, or -1 of the port
- * or service was invalid. */
+ * Returns the port number in _host_ byte order, or 0 if lookup
+ * fails. */
 
 int get_portno(const char *service, const char *protocol)
 {
@@ -858,13 +870,14 @@ int get_portno(const char *service, const char *protocol)
 	  struct servent * serv;
 
 	  serv = getservbyname(service, protocol);
-	  if (serv == NULL)
-	    return -1;
+	  if (!serv)
+	    return 0;
 	  return ntohs(serv->s_port);
 	}
     }
 }
 
+#if 0
 /*
  * Fill in ADDR from HOST, SERVICE and PROTOCOL.
  * Supplying a null pointer for HOST means use INADDR_ANY.
@@ -877,11 +890,12 @@ int get_portno(const char *service, const char *protocol)
  * Returns zero on errors, 1 if everything is ok.
  */
 int
-get_inaddr(struct sockaddr_in	* addr,
+get_inaddr(struct sockaddr	* addr,
 	   const char		* host,
 	   const char		* service,
 	   const char		* protocol)
 {
+  /* HERE!!! IPv6 */
   memset(addr, 0, sizeof *addr);
   addr->sin_family = AF_INET;
 
@@ -979,13 +993,21 @@ int tcp_addr(struct sockaddr_in *sin,
   sin->sin_port = htons(port);
   return 1;
 }
+#endif
 
-struct address_info *make_address_info_c(const char *host,
-					 const char *port)
+/* If def != 0, use that value as a fallback if the lookup fails. */
+struct address_info *
+make_address_info_c(const char *host,
+		    const char *port,
+		    int def)
 {
   int portno = get_portno(port, "tcp");
-  if (portno < 0)
-    return 0;
+  if (!portno)
+    portno = def;
+
+  if (!portno)
+    return NULL;
+
   else
     {
       NEW(address_info, info);
@@ -997,7 +1019,8 @@ struct address_info *make_address_info_c(const char *host,
     }
 }
 
-struct address_info *make_address_info(struct lsh_string *host, UINT32 port)
+struct address_info *
+make_address_info(struct lsh_string *host, UINT32 port)
 {
   NEW(address_info, info);
 
@@ -1006,8 +1029,9 @@ struct address_info *make_address_info(struct lsh_string *host, UINT32 port)
   return info;
 }
 
-struct address_info *sockaddr2info(size_t addr_len UNUSED,
-				   struct sockaddr *addr)
+struct address_info *
+sockaddr2info(size_t addr_len UNUSED,
+	      struct sockaddr *addr)
 {
   NEW(address_info, info);
   
@@ -1025,27 +1049,181 @@ struct address_info *sockaddr2info(size_t addr_len UNUSED,
 			      ip & 0xff);
 	return info;
       }
-#if 0
-    case AF_INETv6:
-      ...
+#if WITH_IPV6
+    case AF_INET6:
+      {
+	struct sockaddr_in6 *in = (struct sockaddr_in6 *) addr;
+	UINT8 *ip = in->sin6_addr.s6_addr;
+	info->port = ntohs(in->sin6_port);
+	info->ip = ssh_format("%xi:%xi:%xi:%xi:%xi:%xi:%xi:%xi",
+			      (ip[0]  << 8) | ip[1],
+			      (ip[2]  << 8) | ip[3],
+			      (ip[4]  << 8) | ip[5],
+			      (ip[6]  << 8) | ip[7],
+			      (ip[8]  << 8) | ip[9],
+			      (ip[10] << 8) | ip[11],
+			      (ip[12] << 8) | ip[13],
+			      (ip[14] << 8) | ip[15]);
+	return info;
+      }
 #endif
     default:
       fatal("io.c: format_addr(): Unsupported address family.\n");
     }
 }
 
-int address_info2sockaddr_in(struct sockaddr_in *sin,
-			     struct address_info *a)
+#if HAVE_GETADDRINFO
+static struct addrinfo *
+choose_address(struct addrinfo *list,
+	       const int *preference)
 {
+  int i;
+  for (i = 0; preference[i]; i++)
+    {
+      struct addrinfo *p;
+      for (p = list; p; p = p->ai_next)
+	if (preference[i] == p->ai_family)
+	  return p;
+    }
+  return NULL;
+}
+#endif /* HAVE_GETADDRINFO */
+
+/* FIXME: Perhaps this function should be changed to return a list of
+ * sockaddr:s? */
+struct sockaddr *
+address_info2sockaddr(socklen_t *length,
+		      struct address_info *a,
+		      /* Preferred address families. Zero-terminated array. */
+		      const int *preference,
+		      int lookup)
+{
+  char *host;
 
   if (a->ip)
-    return tcp_addr(sin,
-		    a->ip->length, a->ip->data,
-		    a->port);
+    {
+      host = alloca(a->ip->length + 1);
+  
+      memcpy(host, a->ip->data, a->ip->length);
+      host[a->ip->length] = '\0';
+    }
   else
-    return tcp_addr(sin, 0, NULL, a->port);
-}
+    host = NULL;
+  
+#if HAVE_GETADDRINFO
+  {
+    struct addrinfo hints;
+    struct addrinfo *list;
+    struct addrinfo *chosen;
+    struct sockaddr *res;
+    const int default_preference
+#if WITH_IPV6
+      [3] = { AF_INET6, AF_INET }
+#else
+      [2] = { AF_INET, 0 }
+#endif      
+      ;
+    int err;
+    /* FIXME: It seems ugly to have to convert the port number to a
+     * string. */
+    struct lsh_string *service = ssh_format("%di%c", a->port, 0);
 
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = PF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_flags = AI_PASSIVE;
+
+    if (!lookup)
+      hints.ai_flags |= AI_NUMERICHOST;
+    
+    err = getaddrinfo(host, service->data, &hints, &list);
+    lsh_string_free(service);
+
+    if (err)
+      {
+	debug("address_info2sockaddr: getaddrinfo failed (err = %d): %s\n",
+	      err, gai_strerror(err));
+	return NULL;
+      }
+
+    chosen = choose_address(list,
+			    preference ? preference : default_preference);
+    if (!chosen)
+      {
+	freeaddrinfo(list);
+	return NULL;
+      }
+    
+    *length = chosen->ai_addrlen;
+    
+    res = lsh_space_alloc(*length);
+    memcpy(res, chosen->ai_addr, *length);
+    freeaddrinfo(list);
+
+    return res;
+  }
+#else /* !HAVE_GETADDRINFO */
+
+#if WITH_IPV6
+#warning IPv6 enabled, but getaddrinfo() was not found. 
+#endif
+
+  if (a->ip && memchr(a->ip->data, ':', a->ip->length))
+    {
+      debug("address_info2sockaddr: Literal IPv6 used. Failing.\n");
+      return NULL;
+    }
+  else
+    {
+      struct sockaddr_in *addr;
+      NEW_SPACE(addr);
+
+      *length = sizeof(*addr);
+      addr->sin_port = htons(a->port);
+    
+      if (!host)
+	{
+	  /* Use INADDR_ANY (and IPv4 only) */
+
+	  addr->sin_family = AF_INET;
+	  addr->sin_addr.s_addr = INADDR_ANY;
+
+	  return (struct sockaddr *) addr;
+	}
+      else
+	{
+	  /* First check for numerical ip-number */
+#if HAVE_INET_ATON
+	  if (!inet_aton(host, &addr->sin_addr))
+#else /* !HAVE_INET_ATON */
+	    /* NOTE: It is wrong to work with ((unsigned long int) -1)
+	     * directly, as this breaks Linux/Alpha systems. But
+	     * INADDR_NONE isn't portable. That's what inet_aton is for;
+	     * see the GNU libc documentation. */
+# ifndef INADDR_NONE
+# define INADDR_NONE ((unsigned long int) -1)
+# endif /* !INADDR_NONE */
+	  addr->sin_addr.s_addr = inet_addr(host);
+	  if (addr->sin_addr.s_addr == INADDR_NONE)
+#endif  /* !HAVE_INET_ATON */
+	    {
+	      struct hostent *hp;
+
+	      if (! (lookup 
+		     && (hp = gethostbyname(host))
+		     && (hp->h_addrtype == AF_INET)))
+		{
+		  lsh_space_free(addr);
+		  return NULL;
+		}
+
+	      memcpy(&addr->sin_addr, hp->h_addr, hp->h_length);
+	    }
+	  return (struct sockaddr *) addr;
+	}
+    }
+#endif /* !HAVE_GETADDRINFO */  
+}
 
 
 void io_set_nonblocking(int fd)
@@ -1100,12 +1278,12 @@ make_lsh_fd(struct io_backend *b,
 /* Some code is taken from Thomas Bellman's tcputils. */
 struct lsh_fd *
 io_connect(struct io_backend *b,
-	   struct sockaddr_in *remote,
-	   struct sockaddr_in *local,
+	   struct sockaddr *remote,
+	   socklen_t remote_length,
 	   struct command_continuation *c,
 	   struct exception_handler *e)
 {
-  int s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
+  int s = socket(remote->sa_family, SOCK_STREAM, 0);
   struct lsh_fd *fd;
   
   if (s<0)
@@ -1115,6 +1293,7 @@ io_connect(struct io_backend *b,
   
   io_init_fd(s);
 
+#if 0
   if (local  &&  bind(s, (struct sockaddr *)local, sizeof *local) < 0)
     {
       int saved_errno = errno;
@@ -1122,8 +1301,9 @@ io_connect(struct io_backend *b,
       errno = saved_errno;
       return NULL;
     }
-
-  if ( (connect(s, (struct sockaddr *)remote, sizeof *remote) < 0)
+#endif
+  
+  if ( (connect(s, remote, remote_length) < 0)
        && (errno != EINPROGRESS) )       
     {
       int saved_errno = errno;
@@ -1142,11 +1322,12 @@ io_connect(struct io_backend *b,
 
 struct lsh_fd *
 io_listen(struct io_backend *b,
-	  struct sockaddr_in *local,
+	  struct sockaddr *local,
+	  socklen_t length,
 	  struct io_callback *callback,
 	  struct exception_handler *e)
 {
-  int s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
+  int s = socket(local->sa_family, SOCK_STREAM, 0);
   struct lsh_fd *fd;
   
   if (s<0)
@@ -1161,7 +1342,7 @@ io_listen(struct io_backend *b,
     setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char*)&yes, sizeof yes);
   }
 
-  if (bind(s, (struct sockaddr *)local, sizeof *local) < 0)
+  if (bind(s, (struct sockaddr *)local, length) < 0)
     {
       close(s);
       return NULL;
@@ -1183,9 +1364,34 @@ io_listen(struct io_backend *b,
 }
 
 
-#if 0
 /* AF_LOCAL sockets */
 
+/* This isn't needed if we have fchdir */
+#if 0
+#define HAVE_GNU_GETCWD 1
+
+/* We have to set and reset the cwd */
+#if HAVE_GNU_GETCWD
+#define gnu_getcwd() getcwd(NULL, 0)
+#else /* !HAVE_GNU_GETCWD */
+char *
+gnu_getcwd ()
+{
+  for (size = 100; size *= 2;)
+    {
+      char *buffer = lsh_space_alloc (size);
+      char *value = getcwd (buffer, size);
+
+      if (value)
+	return buffer;
+
+      lsh_space_free (buffer);
+    }
+}
+#endif /* !HAVE_GNU_GETCWD */
+#endif
+
+/* Requires DIRECTORY and NAME to be NUL-terminated */
 struct lsh_fd *
 io_listen_local(struct io_backend *b,
 		struct lsh_string *directory,
@@ -1193,10 +1399,220 @@ io_listen_local(struct io_backend *b,
 		struct io_callback *callback,
 		struct exception_handler *e)
 {
-  char *old_cd = NULL;
-  mode_t old_umask = umask(); HERE!!!
+  int old_cd;
+
+  mode_t old_umask;
+  struct stat sbuf;
+  struct sockaddr_un *local;
+  socklen_t local_length;
+
+  struct lsh_fd *fd;
+  
+  assert(directory && NUL_TERMINATED(directory));
+  assert(name && NUL_TERMINATED(name));
+
+  /* NAME should not be a plain filename, with no directory separators.
+   * In particular, it should not be an absolute filename. */
+  assert(!memchr(name->data, '/', name->length));
+
+  local_length = OFFSETOF(struct sockaddr_un, sun_path) + name->length;
+  local = alloca(local_length);
+
+  local->sun_family = AF_LOCAL;
+  memcpy(local->sun_path, name->data, name->length);
+  
+  /* First create the directory, in case it doesn't yet exist. */
+  if ( (mkdir(directory->data, 0700) < 0)
+       && (errno != EEXIST) )
+    {
+      werror("io.c: Creating directory %S failed "
+	     "(errno = %i): %z\n", directory, errno, STRERROR(errno));
+    }
+
+  /* cd to it, but first save old cwd */
+
+  old_cd = open(".", O_RDONLY);
+  if (old_cd < 0)
+    {
+      werror("io.c: open(\".\") failed.\n");
+      return NULL;
+    }
+  
+  while ( (chdir(directory->data) < 0) )
+    if (errno != EINTR)
+      {
+	close(old_cd);
+	return NULL;
+      }
+
+  /* Check that it has reasonable permissions */
+  if (stat(".", &sbuf) < 0)
+    {
+      werror("io.c: Failed to stat() \".\" (supposed to be %S).\n"
+	     "  (errno = %i): %z\n", directory, errno, STRERROR(errno));
+
+    fail:
+      if (fchdir(old_cd) < 0)
+	fatal("io_listen_local: Failed to cd back from %S (errno = %i): %z\n",
+	      directory, errno, STRERROR(errno));
+    
+      close(old_cd);
+      return NULL;
+    }
+  
+  if (sbuf.st_uid != getuid())
+    {
+      werror("io.c: Socket directory %S not owned by user.\n", directory);
+      goto fail;
+    }
+
+  if (sbuf.st_mode & (S_IRWXG | S_IRWXO))
+    {
+      werror("io.c: Permission bits on %S are too loose.\n", directory);
+      goto fail;
+    }
+
+  /* Ok, now the current directory seems to be a decent place for
+   * creating a socket. */
+
+  /* Try unlinking any existing file. */
+  if ( (unlink(name->data) < 0)
+       && (errno != ENOENT))
+    {
+      werror("io.c: unlink '%S'/'%S' failed (errno = %i): %z\n",
+	     directory, name, errno, STRERROR(errno));
+      goto fail;
+    }
+
+  /* We have to change the umask, as that's the only way to control
+   * the permissions that bind() uses. */
+
+  old_umask = umask(0770);
+
+  /* Bind and listen */
+  fd = io_listen(b, (struct sockaddr *) local, local_length, callback, e);
+  
+  /* Ok, now we restore umask and cwd */
+  umask(old_umask);
+
+  while (fchdir(old_cd) < 0)
+    if (errno != EINTR)
+      fatal("io_listen_local: Failed to cd back from %S (errno = %i): %z\n",
+	    directory, errno, STRERROR(errno));
+
+  close(old_cd);
+
+  return fd;
+}
+
+/* Requires DIRECTORY and NAME to be NUL-terminated */
+struct lsh_fd *
+io_connect_local(struct io_backend *b,
+		 struct lsh_string *directory,
+		 struct lsh_string *name,
+		 struct command_continuation *c,
+		 struct exception_handler *e)
+{
+  int old_cd;
+
+  struct stat sbuf;
+  struct sockaddr_un *addr;
+  socklen_t addr_length;
+
+  struct lsh_fd *fd;
+  
+  assert(directory && NUL_TERMINATED(directory));
+  assert(name && NUL_TERMINATED(name));
+
+  /* NAME should not be a plain filename, with no directory separators.
+   * In particular, it should not be an absolute filename. */
+  assert(!memchr(name->data, '/', name->length));
+
+  addr_length = OFFSETOF(struct sockaddr_un, sun_path) + name->length;
+  addr = alloca(addr_length);
+
+  addr->sun_family = AF_LOCAL;
+  memcpy(addr->sun_path, name->data, name->length);
+
+  /* cd to it, but first save old cwd */
+
+  old_cd = open(".", O_RDONLY);
+  if (old_cd < 0)
+    {
+      werror("io.c: open(\".\") failed.\n");
+      return NULL;
+    }
+  
+  while ( (chdir(directory->data) < 0) )
+    if (errno != EINTR)
+      {
+	close(old_cd);
+	return NULL;
+      }
+  
+  /* Check that the directory has reasonable permissions */
+  if (stat(".", &sbuf) < 0)
+    {
+      werror("io.c: Failed to stat() \".\" (supposed to be %S).\n"
+	     "  (errno = %i): %z\n", directory, errno, STRERROR(errno));
+
+    fail:
+      if (fchdir(old_cd) < 0)
+	fatal("io_connect_local: Failed to cd back from %S (errno = %i): %z\n",
+	      directory, errno, STRERROR(errno));
+
+      close(old_cd);
+      return NULL;
+    }
+  
+  if (sbuf.st_uid != getuid())
+    {
+      werror("io.c: Socket directory %S not owned by user.\n", directory);
+      goto fail;
+    }
+
+  if (sbuf.st_mode & (S_IRWXG | S_IRWXO))
+    {
+      werror("io.c: Permission bits on %S are too loose.\n", directory);
+      goto fail;
+    }
+
+  /* Check that the socket is owned by the right user */
+  if (stat(name->data, &sbuf) < 0)
+    {
+      werror("io.c: Failed to stat() \"%S/%S\".\n"
+	     "  (errno = %i): %z\n", directory, name, errno, STRERROR(errno));
+
+      goto fail;
+    }
+  if (sbuf.st_uid != getuid())
+    {
+      werror("io.c: Socket %S/%S not owned by user.\n",
+	     directory, name);
+      goto fail;
+    }
+
+  if (sbuf.st_mode & (S_IRWXG | S_IRWXO))
+    {
+      werror("io.c: Permission bits on %S/%S are too loose.\n",
+	     directory, name);
+      goto fail;
+    }
+
+  fd = io_connect(b, (struct sockaddr *) addr, addr_length, c, e);
+
+  while (fchdir(old_cd) < 0)
+    if (errno != EINTR)
+      fatal("io_connect_local: Failed to cd back from %S (errno = %i): %z\n",
+	    directory, errno, STRERROR(errno));
+
+  close(old_cd);
+
+  return fd;
+ 
+  
+  
 }
-#endif
 
 /* Constructors */
 
