@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: publickey_crypto.h,v 1.29 1999/12/12 19:34:05 nisse Exp $ */
+ * $Id: publickey_crypto.h,v 1.30 2000/04/19 00:44:05 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -109,6 +109,11 @@ struct signer *make_dsa_signer_kludge(struct signer *dsa);
 struct signer *make_dsa_signer_classic(struct signer *s);
 struct verifier *make_dsa_verifier_classic(struct verifier *v);
 
+struct signature_algorithm *
+make_rsa_algorithm(struct hash_algorithm *hash,
+		   UINT32 prefix_length,
+		   const UINT8 *prefix);
+
 /* FIXME: Groups could use (meta)class methods */
 
 /* Groups. For now, assume that all group elements are represented by
