@@ -1,6 +1,6 @@
 /* server_session.c
  *
- * $Id: server_session.c,v 1.32 2000/04/12 21:05:49 nisse Exp $
+ * $Id: server_session.c,v 1.36 2000/04/21 22:29:42 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -27,6 +27,7 @@
 #include "channel_commands.h"
 #include "format.h"
 #include "read_data.h"
+#include "reaper.h"
 #include "server_pty.h"
 #include "ssh.h"
 #include "tcpforward.h"
@@ -38,7 +39,7 @@
 #include <errno.h>
 
 /* For debug */
-#include <signal.h>
+
 #include <string.h>
 #if HAVE_UNISTD_H
 #include <unistd.h>
@@ -46,15 +47,7 @@
 
 #include <sys/types.h>
 #include <sys/socket.h>
-/* #include <pwd.h> */
 
-#if HAVE_UTMP_H
-#include <utmp.h>
-#endif
-
-#if HAVE_UTMPX_H
-#include <utmpx.h>
-#endif
 
 /* Socket workround */
 #ifndef SHUTDOWN_WORKS_WITH_UNIX_SOCKETS
@@ -107,47 +100,6 @@
 
 #include "server_session.c.x"
 
-/* GABA:
-   (class
-     (name process_resource)
-     (super resource)
-     (vars
-       (pid . pid_t)
-       (signal . int)))
-*/
-
-static void do_kill_process(struct resource *r)
-{
-  CAST(process_resource, self, r);
-
-  if (self->super.alive)
-    {
-      self->super.alive = 0;
-      /* NOTE: This function only makes one attempt at killing the
-       * process. An improvement would be to install a callout handler
-       * which will kill -9 the process after a delay, if it hasn't died
-       * voluntarily. */
-      
-      if (kill(self->pid, self->signal) < 0)
-	{
-	  werror("do_kill_process: kill() failed (errno = %i): %z\n",
-		 errno, STRERROR(errno));
-	}
-    }
-}
-          
-struct resource *make_process_resource(pid_t pid, int signal)
-{
-  NEW(process_resource, self);
-  self->super.alive = 1;
-
-  self->pid = pid;
-  self->signal = signal;
-
-  self->super.kill = do_kill_process;
-
-  return &self->super;
-}
 
 /* Session */
 /* GABA:
@@ -258,14 +210,12 @@ make_server_session(struct lsh_user *user,
   self->super.rec_window_size = 0;
 
   /* FIXME: Make maximum packet size configurable. */
-  self->super.rec_max_packet = SSH_MAX_PACKET;
-
+  self->super.rec_max_packet = SSH_MAX_PACKET - SSH_CHANNEL_MAX_PACKET_FUZZ;
   self->super.request_types = request_types;
 
   self->super.close = do_close;
   
   self->user = user;
-
   self->process = NULL;
   
   self->in = NULL;
@@ -418,17 +368,6 @@ static void do_exit_shell(struct exit_callback *c, int signaled,
   struct ssh_channel *channel = &session->super;
   
   CHECK_TYPE(server_session, session);
-
-  if (! session->process->alive)
-    {
-      /* The process was killed by a the resource callback (most
-       * likely because the connection died. Keep silent. */
-      debug("do_exit_shell: Process already flagged as dead.\n");
-      return;
-    }
-  
-  /* No need to kill the process. */
-  session->process->alive = 0;
   
   /* FIXME: Should we explicitly mark these files for closing? The
    * io-backend should notice EOF anyway. And the client should send
@@ -476,7 +415,7 @@ make_exit_shell(struct server_session *session)
      (super channel_request)
      (vars
        (backend object io_backend)
-       (reap object reap)))
+       (reaper object reap)))
 */
 
 /* Creates a one-way socket connection. Returns 1 on success, 0 on
@@ -617,8 +556,9 @@ static int make_pty(struct pty_info *pty UNUSED,
 /* Returns -1 on failure, 0 for child and +1 for parent */
 static int
 spawn_process(struct server_session *session,
+	      struct address_info *peer,
 	      struct io_backend *backend,
-	      struct reap *reap)
+	      struct reap *reaper)
 {
   int in[2];
   int out[2];
@@ -638,18 +578,21 @@ spawn_process(struct server_session *session,
 
   else if (!make_pipes(in, out, err))
     return -1;
-  {
-    pid_t child;
 
-    if (USER_FORK(session->user, &child))
+  {
+    struct resource *child;
+    
+    if (USER_FORK(session->user, &child,
+		  reaper, make_exit_shell(session),
+		  peer, using_pty ? session->pty->tty_name : NULL))
       {
 	if (child)
 	  { /* Parent */
 	    struct ssh_channel *channel = &session->super;
-	    
 	    debug("Parent process\n");
-	    REAP(reap, child, make_exit_shell(session));
-	  
+
+	    session->process = child;
+
 	    /* Close the child's fd:s */
 	    close(in[0]);
 	    close(out[1]);
@@ -694,13 +637,10 @@ spawn_process(struct server_session *session,
 	    channel->send_adjust = do_send_adjust;
 	    channel->eof = do_eof;
 	  
-	    session->process
-	      = make_process_resource(child, SIGHUP);
-
 	    /* Make sure that the process and it's stdio is
 	     * cleaned up if the channel or connection dies. */
 	    REMEMBER_RESOURCE
-	      (channel->resources, session->process);
+	      (channel->resources, child);
 	    /* FIXME: How to do this properly if in and out may use the
 	     * same fd? */
 	    REMEMBER_RESOURCE
@@ -797,7 +737,7 @@ STATIC_EXCEPTION(EXC_CHANNEL_REQUEST, "Shell request failed");
 static void
 do_spawn_shell(struct channel_request *c,
 	       struct ssh_channel *channel,
-	       struct ssh_connection *connection UNUSED,
+	       struct ssh_connection *connection,
 	       UINT32 type UNUSED,
 	       int want_reply UNUSED,
 	       struct simple_buffer *args,
@@ -817,7 +757,8 @@ do_spawn_shell(struct channel_request *c,
     /* Already spawned a shell or command */
     goto fail;
 
-  switch (spawn_process(session, closure->backend, closure->reap))
+  switch (spawn_process(session, connection->peer,
+			closure->backend, closure->reaper))
     {
     case 1: /* Parent */
       /* NOTE: The return value is not used. */
@@ -877,13 +818,13 @@ do_spawn_shell(struct channel_request *c,
 
 struct channel_request *
 make_shell_handler(struct io_backend *backend,
-		   struct reap *reap)
+		   struct reap *reaper)
 {
   NEW(shell_request, closure);
 
   closure->super.handler = do_spawn_shell;
   closure->backend = backend;
-  closure->reap = reap;
+  closure->reaper = reaper;
   
   return &closure->super;
 }
@@ -894,7 +835,7 @@ STATIC_EXCEPTION(EXC_CHANNEL_REQUEST, "Exec request failed");
 static void
 do_spawn_exec(struct channel_request *c,
 	      struct ssh_channel *channel,
-	      struct ssh_connection *connection UNUSED,
+	      struct ssh_connection *connection,
 	      UINT32 type UNUSED,
 	      int want_reply UNUSED,
 	      struct simple_buffer *args,
@@ -924,7 +865,8 @@ do_spawn_exec(struct channel_request *c,
   if (!command_line)
     EXCEPTION_RAISE(e, &exec_request_failed);
   else
-    switch (spawn_process(session, closure->backend, closure->reap))
+    switch (spawn_process(session, connection->peer,
+			  closure->backend, closure->reaper))
     {
     case 1: /* Parent */
       lsh_string_free(command_line);
@@ -975,13 +917,13 @@ do_spawn_exec(struct channel_request *c,
 
 struct channel_request *
 make_exec_handler(struct io_backend *backend,
-		  struct reap *reap)
+		  struct reap *reaper)
 {
   NEW(shell_request, closure);
 
   closure->super.handler = do_spawn_exec;
   closure->backend = backend;
-  closure->reap = reap;
+  closure->reaper = reaper;
   
   return &closure->super;
 }
