@@ -1,6 +1,6 @@
 /* server_session.h
  *
- * $Id: server_session.h,v 1.8 2000/04/02 19:33:30 nisse Exp $
+ * $Id: server_session.h,v 1.11 2000/04/21 22:29:42 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -27,7 +27,6 @@
 
 #include "channel.h"
 #include "io.h"
-#include "reaper.h"
 #include "server_userauth.h"
 
 #include <assert.h>
@@ -46,11 +45,11 @@ make_open_session(struct lsh_user *user,
 
 struct channel_request *
 make_shell_handler(struct io_backend *backend,
-		   struct reap *reap);
+		   struct reap *reaper);
 
 struct channel_request *
 make_exec_handler(struct io_backend *backend,
-		  struct reap *reap);
+		  struct reap *reaper);
 
 struct lsh_string *
 format_exit_signal(struct ssh_channel *channel,
@@ -58,9 +57,6 @@ format_exit_signal(struct ssh_channel *channel,
 struct lsh_string *
 format_exit(struct ssh_channel *channel, int value);
 
-struct resource *
-make_process_resource(pid_t pid, int signal);
-
 struct channel_request *
 make_pty_handler(void);
 
